"""Routes configuration

The more specific and detailed routes should be defined first so they
may take precedent over the more generic routes. For more information
refer to the routes manual at http://routes.groovie.org/docs/
"""
from pylons import config
from routes import Mapper

def make_map():
    """Create, configure and return the routes Mapper"""
    map = Mapper(directory=config['pylons.paths']['controllers'], always_scan=config['debug'])
    map.minimization = False
    
    # The ErrorController route (handles 404/500 error pages); it should
    # likely stay at the top, ensuring it can always be resolved
    map.connect('/error/{action}', controller='error')
    map.connect('/error/{action}/{id}', controller='error')
    
    # CUSTOM ROUTES HERE
    
    valid_name = config['path.valid_name']
    if valid_name.startswith('^'):
        valid_name = valid_name[1:]
    if valid_name.endswith('$'):
        valid_name = valid_name[:-1]
        
    valid_language = config['path.valid_language']
    if valid_language.startswith('^'):
        valid_language = valid_language[1:]
    if valid_language.endswith('$'):
        valid_language = valid_language[:-1]
        
    map.connect('/', controller='root')
    map.connect('/openid_login', controller='openid_handler', action='login')
    map.connect('/openid_logout', controller='openid_handler', action='logout')
    map.connect('/openid_verified', controller='openid_handler', action='verified')
    
    map.connect('/manage_admin', controller='manage_server')
    map.connect('/manage_admin_create', controller='manage_server', action='create')
    map.connect('/manage_admin_grant', controller='manage_server', action='grant')
    map.connect('/manage_admin_revoke', controller='manage_server', action='revoke')
    
    map.connect('/{project_family:%(name)s}' % {
     'name': valid_name,
    }, controller='root_project_family')
    map.connect('/{project_family:%(name)s}/manage_settings' % {
     'name': valid_name,
    }, controller='manage_project_family')
    map.connect('/{project_family:%(name)s}/manage_settings_create' % {
     'name': valid_name,
    }, controller='manage_project_family', action='create')
    map.connect('/{project_family:%(name)s}/manage_settings_owner_grant' % {
     'name': valid_name,
    }, controller='manage_project_family', action='owner_grant')
    map.connect('/{project_family:%(name)s}/manage_settings_owner_revoke' % {
     'name': valid_name,
    }, controller='manage_project_family', action='owner_revoke')
    map.connect('/{project_family:%(name)s}/manage_settings_settings' % {
     'name': valid_name,
    }, controller='manage_project_family', action='settings')
    map.connect('/{project_family:%(name)s}/manage_settings_language_add' % {
     'name': valid_name,
    }, controller='manage_project_family', action='language_add')
    map.connect('/{project_family:%(name)s}/manage_settings_language_remove' % {
     'name': valid_name,
    }, controller='manage_project_family', action='language_remove')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}' % {
     'name': valid_name,
    }, controller='root_project')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/manage_settings' % {
     'name': valid_name,
    }, controller='manage_project')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/manage_settings_settings' % {
     'name': valid_name,
    }, controller='manage_project', action='settings')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/manage_settings_screenshot_create' % {
     'name': valid_name,
    }, controller='manage_project', action='screenshot_create')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/manage_settings_screenshot_modify' % {
     'name': valid_name,
    }, controller='manage_project', action='screenshot_modify')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/manage_settings_language_add' % {
     'name': valid_name,
    }, controller='manage_project', action='language_add')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/manage_settings_language_remove' % {
     'name': valid_name,
    }, controller='manage_project', action='language_remove')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{contributors:(.*?)\.csv}' % {
     'name': valid_name,
    }, controller='screenshot_contributors')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{qsproj:(.*?)\.qsproj}' % {
     'name': valid_name,
    }, controller='screenshot_project')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='root_project_translation')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/manage_translations' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='manage_project_translation')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/manage_translations_translate' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='manage_project_translation', action="translate")
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/{screenshot:%(name)s}' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='manage_project_screenshot_approval')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/{screenshot:%(name)s}/accept' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='manage_project_screenshot_approval', action='accept')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/{screenshot:%(name)s}/reject' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='manage_project_screenshot_approval', action='reject')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/_upload' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='screenshot_upload')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/_progress' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='screenshot_progress')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{screenshot:%(name)s}/_details' % {
     'name': valid_name,
    }, controller='screenshot_details')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{screenshot:%(name)s}/_reference' % {
     'name': valid_name,
    }, controller='screenshot_download', action='reference')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/{screenshot:%(name)s}/_latest' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='screenshot_download', action='latest')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/{screenshot:%(name)s}/_accepted' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='screenshot_download', action='accepted')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/{screenshot:%(name)s}/_pending/{timestamp:\d+}' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='screenshot_download', action='pending')
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/{screenshot:%(name)s}/_rejected/{timestamp:\d+}' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='screenshot_download', action='rejected')
    
    map.connect('/{project_family:%(name)s}/{project:%(name)s}/{language:%(language)s}/{archive:(.*?)\.tar}' % {
     'name': valid_name,
     'language': valid_language,
    }, controller='screenshot_archive')
    
    #A simple rule to discard trailing slashes, 'cause it makes things easier for users.
    #http://stackoverflow.com/questions/235191/trailing-slashes-in-pylons-routes/1441104#1441104
    map.redirect('/*(url)/', '/{url}', _redirect_code='301 Moved Permanently')
    
    return map
    
