import logging
import re

from pylons import config, url, request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from sqlalchemy.exc import IntegrityError
import sqlalchemy.sql

from quickshot.lib.base import BaseController, render
import quickshot.lib.helpers as h
import quickshot.model as model
import quickshot.model.meta as meta

log = logging.getLogger(__name__)

class ManageProjectTranslationController(BaseController):
    def index(self, project_family, project, language):
        if not self._load_information(project_family, project, language):
            h.unauthorized()
            
        c.language = language
        
        return render('/manage_project_translation.mako')
        
    def translate(self, project_family, project, language):
        if not self._load_information(project_family, project, language):
            h.unauthorized()
            
        delete_translation = bool(request.params.get('delete-translation', False))
        friendly_name = request.params.get('friendly-name', '').strip()
        screenshot_name = request.params.get('screenshot-name', '').strip()
        if (not friendly_name and not delete_translation) or not screenshot_name:
            details = []
            if not friendly_name and not delete_translation:
                details.append("Friendly name not supplied")
            if not screenshot_name:
                details.append("Screenshot name not supplied")
                
            session['problem_report'] = (
             ("Insufficient information provided", details),
            )
            session.save()
            redirect_to(c.current_page)
            
        if not self._check_screenshot_name(screenshot_name):
            redirect_to(c.current_page)
            
        translation_query = meta.Session.query(
         model.ProjectScreenshotTranslation
        ).filter(
         model.ProjectScreenshotTranslation.ps_short_name == project_family
        ).filter(
         model.ProjectScreenshotTranslation.ps_version == project
        ).filter(
         model.ProjectScreenshotTranslation.ps_screenshot_name == screenshot_name
        ).filter(
         model.ProjectScreenshotTranslation.language_code == language
        )
        
        if delete_translation:
            translation_query.delete()
            
            try:
                meta.Session.commit()
            except Exception, e:
                meta.Session.rollback()
                
                details = [h.screen_error(e)]
                session['problem_report'] = (
                 ("Unable to delete translation", details),
                )
            else:
                session['success_report'] = (
                 ("Deleted translation", (
                  "Screenshot name: %(screenshot-name)s" % {
                   'screenshot-name': screenshot_name,
                  },
                 )),
                )
            finally:
                session.save()
        else:
            translation = translation_query.first()
            new_entry = not translation
            if new_entry:
                translation = model.ProjectScreenshotTranslation()
                translation.ps_short_name = project_family
                translation.ps_version = project
                translation.ps_screenshot_name = screenshot_name
                translation.language_code = language
                
            translation.friendly_name = unicode(friendly_name)
            translation.steps = unicode('\n'.join([line.strip() for line in request.params.get('steps', '').splitlines()]))
            
            if new_entry:
                meta.Session.add(translation)
                
            try:
                meta.Session.commit()
            except Exception, e:
                meta.Session.rollback()
                
                details = []
                if type(e) is IntegrityError:
                    details.append("The given translation already exists")
                else:
                    details.append(h.screen_error(e))
                    
                session['problem_report'] = (
                 ("Unable to update translation", details),
                )
            else:
                session['success_report'] = (
                 ("Updated translation", (
                  "Friendly name: %(friendly-name)s" % {
                   'friendly-name': friendly_name,
                  },
                  "Screenshot name: %(screenshot-name)s" % {
                   'screenshot-name': screenshot_name,
                  },
                  "Steps: %(steps)s" % {
                   'steps': repr(translation.steps or translation.screenshot.steps),
                  },
                 )),
                )
            finally:
                session.save()
        redirect_to(c.current_page)
        
    def _check_screenshot_name(self, screenshot_name):
        if not re.match(config['path.valid_name'], screenshot_name):
            session['problem_report'] = (
             ("Invalid screenshot name", (
              "Screenshot names must be all-lower-case, start with an alphanumeric, and use only '-' as a separator (%(regexp)s)" % {
               'regexp': config['path.valid_name'],
              },
             )),
            )
            session.save()
            return False
        return True
        
    def _load_information(self, project_family, project, language):
        c.current_page = url(str(''.join(('/', project_family, '/', project, '/', language, '/manage_translations'))))
        c.admin = h.assert_owner(project_family)
        c.project = h.get_project_details(project_family, project)
        
        return c.project and c.admin
        
