"""
This logic is based on Turophile's "OpenID Login 0.9.6 w/o AuthKit" from
http://pylonshq.com/pasties/399
"""
import logging
import os
import sqlite3

from openid.store.sqlstore import SQLiteStore
from openid.consumer.consumer import Consumer, SUCCESS, DiscoveryFailure
from openid.extensions import sreg

from pylons import config, url, request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from quickshot.lib.base import BaseController, render
import quickshot.lib.helpers as h

log = logging.getLogger(__name__)

if not os.path.exists(config['openid.store_path']):
    conn = sqlite3.connect(config['openid.store_path'])
    openid_store = SQLiteStore(conn)
    openid_store.createTables()
    conn.close()
    
class OpenidHandlerController(BaseController):
    def __before__(self):
        self._conn = sqlite3.connect(config['openid.store_path'])
        self._openid_store = SQLiteStore(self._conn)
        self._openid_store.cleanup()
        
        self._openid_session = session.get('openid-session', {})
        
    def __after__(self):
        self._conn.commit()
        self._conn.close()
        
    def login(self):
        self._consumer = Consumer(self._openid_session, self._openid_store)
        
        openid = request.params.get('openid_url', None)
        if not openid or not openid.strip():
            session['problem_report'] = (
             ("No OpenID supplied", ()),
            )
            session.save()
            return h.last_visited_page()
            
        try:
            openid = h.normalise_openid(openid)
        except ValueError, e:
            session['problem_report'] = (
             ("The OpenID you provided was in a format that could not be understood", (
              str(e),
             )),
            )
            session.save()
            return h.last_visited_page()
            
        try:
            auth_request = self._consumer.begin(openid)
        except DiscoveryFailure, e:
            session['problem_report'] = (
             ("No OpenID endpoints found", (
              "The address you provided, %(address)s, does not appear to be a valid OpenID provider" % {
               'address': openid,
              },
             )),
            )
            session.save()
            return h.last_visited_page()
        auth_request.addExtension(sreg.SRegRequest(optional=['nickname', 'fullname',]))
        
        session['openid-session'] = self._openid_session
        session.save()
        
        redirect_to(auth_request.redirectURL(
         ''.join((request.host_url, url('/'))),
         return_to=(''.join((request.host_url, url('/openid_verified'))))
        ))
        
    def verified(self):
        self._consumer = Consumer(self._openid_session, self._openid_store)
        
        info = self._consumer.complete(
         request.params,
         (''.join((request.host_url, url('/openid_verified'))))
        )
        if info.status == SUCCESS:
            #Store the canonicalized OpenID string in the user's session.
            try:
                session['openid'] = unicode(h.normalise_openid(info.identity_url))
            except ValueError, e:
                session['problem_report'] = (
                 ("Your provider returned an OpenID that could not be understood", (
                  (hasattr('message', info) and info.message) or 'Specific details unavailable',
                  str(e),
                 )),
                )
            if 'openid-session' in session:
                del session['openid-session']
                
            #Attempt to show the user prettier information.
            sreg_details = sreg.SRegResponse.fromSuccessResponse(info)
            if sreg_details:
                session['openid-display'] = sreg_details.get('fullname') or sreg_details.get('nickname')
                if session['openid-display'] and not session['openid-display'].strip():
                    del session['openid-display']
        else:
            session['problem_report'] = (
             ("Your provider was not able to authenticate your OpenID", ()),
            )
        session.save()
        return h.last_visited_page()
        
    def logout(self):
        if 'openid' in session:
            del session['openid']
        if 'openid-display' in session:
            del session['openid-display']
        if 'openid-session' in session:
            del session['openid-session']
        session.save()
        
        return h.last_visited_page()
        
