import logging

from pylons import url, request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from quickshot.lib.base import BaseController, render
import quickshot.lib.helpers as h
import quickshot.model as model
import quickshot.model.meta as meta

log = logging.getLogger(__name__)

class RootProjectTranslationController(BaseController):
    def index(self, project_family, project, language):
        if not self._load_information(project_family, project):
            h.unauthorized()
            
        if not language in c.project.all_languages:
            h.unsupported()
        c.language = language
        
        c.screenshots_accepted = [screenshot_name for (screenshot_name, filenames) in h.list_screenshots_accepted(c.project, language)]
        c.screenshots_pending = [screenshot_name for (screenshot_name, filenames) in h.list_screenshots_pending(c.project, language)]
        c.screenshots = [screenshot.screenshot_name for screenshot in c.project.active_screenshots]
        unneeded = set(c.screenshots_pending + c.screenshots_accepted)
        c.screenshots_needed = [screenshot for screenshot in c.screenshots if not screenshot in unneeded]
        
        return render('/root_project_translation.mako')
        
    def _load_information(self, project_family, project):
        c.current_page = url(str(''.join(('/', project_family, '/', project))))
        c.admin = h.assert_owner(project_family)
        c.project = h.get_project_details(project_family, project)
        
        return c.project and c.admin
        
