import logging
import os
import tarfile
import tempfile

from pylons import config, request, response
from pylons.controllers.util import abort

from quickshot.lib.base import BaseController
import quickshot.lib.helpers as h

log = logging.getLogger(__name__)

class ScreenshotArchiveController(BaseController):
    def index(self, project_family, project, language, archive):
        """
        @type archive: basestr
        @param archive: Not actually used.
        """
        project_details = h.get_project_details(project_family, project)
        
        if not project_details or not project_details.is_active or not h.assert_owner(project_family):
            abort(404, "Project not found")
        response.content_type = 'application/x-tar'
        
        output = tempfile.TemporaryFile()
        tarball = tarfile.open(fileobj=output, mode='w')
        
        base_path = os.path.join(h.get_screenshots_base_path(project_details, language), config['quickshot.data_state_accepted']) + '/'
        tarfile_path = '[' + project_family + ']' + project + '/' + language + '/'
        for (screenshot_name, screenshot_filenames) in h.list_screenshots_accepted(project_details, language):
            screenshot_filename = screenshot_filenames[0]
            tarball.add(base_path + screenshot_filename, tarfile_path + screenshot_filename)
        tarball.close() #Does not close output: http://docs.python.org/library/tarfile.html#tarfile-objects
        
        return h.serve_file(output)
        
