import csv
import logging

from pylons import config, request, response
from pylons.controllers.util import abort

from quickshot.lib.base import BaseController
import quickshot.lib.helpers as h

log = logging.getLogger(__name__)

class ScreenshotContributorsController(BaseController):
    def index(self, project_family, project, contributors):
        """
        @type contributors: basestr
        @param contributors: Not actually used.
        """
        project_details = h.get_project_details(project_family, project)
        
        if not project_details or not project_details.is_active or not h.assert_owner(project_family):
            abort(404, "Project not found")
        response.content_type = 'text/csv'
        
        class _Output(object):
            def __init__(self):
                self._buffer = []
                
            def __str__(self):
                return ''.join(self._buffer)
                
            def write(self, line):
                self._buffer.append(line)
                
        output = _Output()
        csv_output = csv.writer(output)
        
        csv_output.writerow((
         'anonymous',
         'display',
         'accepted',
         'e-mail',
         'name',
         'homepage',
        ))
        
        for contributor in project_details.contributors:
            display = ''
            if contributor.uploader_publish:
                if contributor.uploader_name:
                    if contributor.uploader_homepage:
                        display = contributor.uploader_name + ', ' + contributor.uploader_homepage
                    else:
                        display = contributor.uploader_name
                elif contributor.uploader_homepage:
                    display = contributor.uploader_homepage
                else:
                    display = contributor.uploader
                    
            csv_output.writerow((
             (contributor.uploader_publish and 'no') or 'yes',
             display,
             contributor.submission_count,
             contributor.uploader,
             contributor.uploader_name or '',
             contributor.uploader_homepage or '',
            ))
            
        return str(output)
        
