import logging

from pylons import config, request, response
from pylons.controllers.util import abort

from quickshot.lib.base import BaseController
import quickshot.lib.helpers as h
import quickshot.model as model
import quickshot.model.meta as meta

log = logging.getLogger(__name__)

class ScreenshotDetailsController(BaseController):
    def index(self, project_family, project, screenshot):
        project_details = h.get_project_details(project_family, project)
        
        if not project_details or not project_details.is_active:
            abort(404, "Project not found")
        response.content_type = 'text/plain'
        
        #Retrieve screenshot details.
        details = h.get_screenshot(project_details, screenshot)
        if not details or not details.active:
            return h.serve_error(
             "Invalid screenshot requested",
             "%(project-family)s/%(project)s does not currently accept '%(screenshot)s'" % {
              'project-family': project_family,
              'project': project,
              'screenshot': screenshot,
             }
            )
            
        translation = h.get_screenshot_translation(details, request.params.get('interface-language', 'INVALID'))
        
        #Build response.
        summary = [
         'summary\t' + ((translation and translation.friendly_name) or details.friendly_name),
         'steps\t' + '\t'.join(((translation and translation.steps) or details.steps).splitlines()),
        ]
        if details.command:
            summary.append('command\t' + '\t'.join(details.command.splitlines()))
        if details.maximize:
            summary.append('maximize')
            if details.maximize_workaround:
                summary.append('maximize-workaround\t' + details.maximize_workaround)
        if details.fullscreen:
            summary.append('fullscreen')
        if not None in (details.sub_rect_left, details.sub_rect_top, details.sub_rect_width, details.sub_rect_height):
            summary.append('sub-rect\t' + ':'.join([str(i) for i in (details.sub_rect_left, details.sub_rect_top, details.sub_rect_width, details.sub_rect_height)]))
            if details.sub_rect_invert_x:
                summary.append('sub-rect-invert-x')
            if details.sub_rect_invert_y:
                summary.append('sub-rect-invert-y')
                
        #Done.
        return h.serve_success(
         "%(screenshot)s details" % {'screenshot': screenshot,},
         '\n'.join(summary)
        )
        
