import imghdr
import logging
import os
import tempfile

from PIL import Image

from pylons import config, request, response
from pylons.controllers.util import abort

from quickshot.lib.base import BaseController
import quickshot.lib.helpers as h

log = logging.getLogger(__name__)

class ScreenshotDownloadController(BaseController):
    def reference(self, project_family, project, screenshot):
        details = self._get_project_details(project_family, project)
        interface_language = unicode(request.params.get('interface-language', 'en'))
        path = h.find_reference_screenshot(details, interface_language, screenshot)
        return self._render_screenshot(path)
        
    def latest(self, project_family, project, language, screenshot):
        details = self._get_project_details(project_family, project)
        path = h.find_latest_screenshot(details, language, screenshot)
        return self._render_screenshot(path)
        
    def accepted(self, project_family, project, language, screenshot):
        details = self._get_project_details(project_family, project)
        path = h.find_accepted_screenshot(details, language, screenshot)
        return self._render_screenshot(path)
        
    def pending(self, project_family, project, language, screenshot, timestamp):
        details = self._get_project_details(project_family, project)
        path = os.path.join(
         h.get_screenshots_base_path(details, language=language),
         config['quickshot.data_state_pending'],
         "%(screenshot)s@%(timestamp)s.%(extension)s" % {
          'screenshot': screenshot,
          'timestamp': timestamp,
          'extension': config['quickshot.screenshot_type'],
         }
        )
        return self._render_screenshot(path)
        
    def rejected(self, project_family, project, language, screenshot, timestamp):
        details = self._get_project_details(project_family, project)
        path = os.path.join(
         h.get_screenshots_base_path(details, language=language),
         config['quickshot.data_state_rejected'],
         "%(screenshot)s@%(timestamp)s.%(extension)s" % {
          'screenshot': screenshot,
          'timestamp': timestamp,
          'extension': config['quickshot.screenshot_type'],
         }
        )
        return self._render_screenshot(path)
        
    def _get_project_details(self, project_family, project):
        project_details = h.get_project_details(project_family, project)
        if not project_details:
            abort(404, "Project not found")
        return project_details
        
    def _render_screenshot(self, path):
        if not path or not os.path.isfile(path):
            abort(404, "Screenshot not found")
            
        if request.params: #Engage the processing engine.
            try:
                screenshot = Image.open(path)
            except Exception:
                abort(503, "Screenshot not unusable - not an image or unreadable")
            else:
                if not screenshot.mode == "RGB":
                    screenshot = screenshot.convert("RGB")
                    
                width = request.params.get('thumbnail-width', '')
                height = request.params.get('thumbnail-height', '')
                if width.isdigit() and height.isdigit():
                    screenshot.thumbnail((
                     max(1, int(width)),
                     max(1, int(height))
                    ), Image.ANTIALIAS)
                    
                output = tempfile.SpooledTemporaryFile(max_size=512*1024)
                compression = request.params.get('quality', '')
                if compression.isdigit():
                    response.content_type = 'image/jpeg'
                    screenshot.save(
                     output, 'jpeg',
                     quality=max(1, min(95, int(compression)))
                    )
                else:
                    response.content_type = 'image/png'
                    screenshot.save(
                     output, 'png',
                    )
                    
                del screenshot
                return h.serve_file(output)
        else: #Just pass the file directly.
            try:
                screenshot = open(path, 'rb')
            except Exception:
                abort(503, "Screenshot not unusable - file unreadable")
            else:
                image_type = imghdr.what('', screenshot.read(4096))
                if not image_type:
                    abort(503, "Screenshot not unusable - not an image")
                    
                response.content_type = 'image/' + image_type
                return h.serve_file(screenshot)
                
