import logging
import mimetools

from pylons import config, request, response
from pylons.controllers.util import abort

from quickshot.lib.base import BaseController
import quickshot.lib.helpers as h

log = logging.getLogger(__name__)

class ScreenshotProgressController(BaseController):
    def index(self, project_family, project, language):
        project_details = h.get_project_details(project_family, project)
        
        if not project_details or not project_details.is_active:
            abort(404, "Project not found")
        response.content_type = 'text/plain'
        
        #Validate language.
        language_code = project_details.get_best_language(language)
        if not language_code:
            return h.serve_error(
             "Invalid language code provided",
             "%(project-family)s/%(project)s does not currently accept screenshots for '%(language)s'" % {
              'project-family': project_family,
              'project': project,
              'language': language,
             }
            )
        interface_language = unicode(request.params.get('interface-language', language_code))
        
        #Scan filesystem.
        pending = set([screenshot_name for (screenshot_name, filenames) in h.list_screenshots_pending(project_details, language_code)])
        accepted = set([screenshot_name for (screenshot_name, filenames) in h.list_screenshots_accepted(project_details, language_code)])
        
        #Build response.
        screenshot_progress = []
        for screenshot in project_details.active_screenshots:
            state = config['quickshot.data_state_needed']
            if screenshot.screenshot_name in pending:
                state = config['quickshot.data_state_pending']
            elif screenshot.screenshot_name in accepted:
                state = config['quickshot.data_state_accepted']
                
            translation = [t for t in screenshot.translations if t.language_code == interface_language]
            friendly_name = (translation and translation[0].friendly_name) or screenshot.friendly_name
            
            screenshot_progress.append((state, screenshot.screenshot_name, friendly_name))
            
        #Done.
        return h.serve_success(
         "Screenshot progress for %(language)s" % {'language': language_code,},
         '\n'.join(['\t'.join(screenshot_details) for screenshot_details in screenshot_progress])
        )
        
