import logging

from pylons import config, request, response
from pylons.controllers.util import abort

from quickshot.lib.base import BaseController
import quickshot.lib.helpers as h
import quickshot.model as model
import quickshot.model.meta as meta

log = logging.getLogger(__name__)

_PROTOCOL_VERSION = '0-gamma' #: The version of the Quickshot protocol understood by this server.

class ScreenshotProjectController(BaseController):
    def index(self, project_family, project):
        project_details = h.get_project_details(project_family, project)
        
        if not project_details or (not project_details.is_active and not h.assert_owner(project_family)):
            abort(404, "Project not found")
        response.content_type = 'text/plain'
        
        language = request.params.get('language', None)
        if language and not project_details.get_best_language(language):
            abort(400, "QS-SUMMARY: %(project-family)s/%(project)s does not currently accept screenshots for '%(language)s'" % {
             'project-family': project_family,
             'project': project,
             'language': language or 'language-not-specified',
            })
            
        #Build response.
        output = [
         '#Save this file and open it with Quickshot to help out!',
         '',
         '[system]',
         'url=' + request.url[:request.url.rfind('/')],
         'version=' + _PROTOCOL_VERSION,
         'public=' + str(project_details.is_public),
         '',
         '[meta]',
         'project-friendly-name=' + project_details.project_family.friendly_name,
         'project-homepage=' + (project_details.project_family.homepage or ''),
         'version-friendly-name=' + project_details.friendly_name,
         'version-homepage=' + (project_details.homepage or project_details.project_family.homepage or ''),
         'expiration-date=' + ((project_details.expiration_date and '%(year)s-%(month)s-%(day)s' % {
          'year': project_details.expiration_date.year,
          'month': project_details.expiration_date.month,
          'day': project_details.expiration_date.day,
         }) or ''),
         'languages=' + ','.join(sorted(project_details.all_languages)),
         '',
         '[config]',
         'resolution-x=' + str(max(0, project_details.resolution_x)),
         'resolution-y=' + str(max(0, project_details.resolution_y)),
         'username=' + (project_details.username or ''),
         'display-name=' + (project_details.display_name or ''),
        ]
        
        #Done.
        return '\n'.join(output)
        
