#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Stuff

Presumes SQLite as the database.
"""
import ConfigParser
import os
import sqlite3
import sys

def fail(message):
    sys.stderr.write(message + '\n')
    exit(1)
    
current_path = os.path.abspath(os.path.dirname(sys.argv[0])) + os.path.sep + '..'
project_family = len(sys.argv) > 1 and sys.argv[1]
project = len(sys.argv) > 2 and sys.argv[2]
if not project:
    fail("The first and second arguments to this script must be the short-name of the project to remove and the version, respectively")
    
config_details = ConfigParser.RawConfigParser()
if not config_details.read(current_path + os.path.sep + 'production.ini'):
    fail("'production.ini' not found in parent path; unable to determine data sources")
    
if not config_details.has_option('app:main', 'quickshot.data_path'):
    fail("Screenshot path not defined")
    
if not config_details.has_option('app:main', 'sqlalchemy.url'):
    fail("SQLite path not defined")
    
screenshot_path = config_details.get('app:main', 'quickshot.data_path') % {'here': current_path,}
screenshot_path += os.path.sep + project_family + os.path.sep + project
database_path = config_details.get('app:main', 'sqlalchemy.url')[9:] % {'here': current_path,}

if os.path.isdir(screenshot_path) and not os.access(screenshot_path, os.F_OK | os.R_OK | os.W_OK):
    fail("You do not have sufficient access to the screenshot path, " + screenshot_path)
    
if not os.access(database_path, os.F_OK | os.R_OK | os.W_OK):
    fail("You do not have sufficient access to the database path, " + database_path)
    
print "You are about to remove %(project_family)s/%(project)s. If this is what you want, type 'okay' to proceed:" % {
 'project_family': project_family,
 'project': project,
},
if not raw_input().lower() == 'okay':
    print "Exiting without removing content"
    exit(0)
    
if os.path.isdir(screenshot_path):
    os.unlink(screenshot_path)
    print "Unlinked " + screenshot_path
else:
    print screenshot_path + " did not need to be unlinked"
    
db_connection = sqlite3.connect(database_path)
cursor = db_connection.cursor()

cursor.execute("DELETE FROM project_screenshot_approvals WHERE p_short_name = ? AND p_version = ?", (project_family, project))
cursor.execute("DELETE FROM project_screenshot_uploads WHERE p_short_name = ? AND p_version = ?", (project_family, project))
cursor.execute("DELETE FROM project_screenshot_translations WHERE ps_short_name = ? AND ps_version = ?", (project_family, project))
cursor.execute("DELETE FROM project_screenshots WHERE p_short_name = ? AND p_version = ?", (project_family, project))
cursor.execute("DELETE FROM project_languages WHERE p_short_name = ? AND p_version = ?", (project_family, project))
cursor.execute("DELETE FROM projects WHERE pf_short_name = ? AND version = ?", (project_family, project))

db_connection.commit()
cursor.close()
db_connection.close()
print "Cleared database"

